unit U_8QueensPlus1;
 {Copyright   2001-2004, Gary Darby,  www.DelphiForFun.org
 This program may be used or modified for any non-commercial purpose
 so long as this original notice remains in place.
 All other rights are reserved
 }

{Problem is to place 8 tokens on a chess board so that no 2 are in
line.  The Plus version also requires that no tokens are on the main diagonals.

I defined Board as an 8X8 array of integers to represent
the chessboard.  Board is initialized with 0's to indicate and
available position.  For the Plus version, -1's mark forbidden positions
(the main diagonals).  For placed tokens, the token number to indicate its token.

PlaceCounter(n) searches recursively for a position for queen n by
searching all possible locations. Functions "RowisClear",
"ColIsClear" and "DiagsAreClear" check that a token can be legally
placed and tries to place the queen N+1 if a place is found (if n <8).
If queen cannot be placed, then return false.   When all have been tried,
the final result is returned up the line.

}


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, Buttons, ExtCtrls, ShellAPI;

type
  TBoard=array[1..8,1..8] of integer;
  TForm1 = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    StringGrid1: TStringGrid;
    BitBtn1: TBitBtn;
    Memo1: TMemo;
    Typegrp: TRadioGroup;
    StaticText1: TStaticText;
    procedure SolveBtn(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure StaticText1Click(Sender: TObject);
  public
    board:TBoard;
    counter:integer;
    Function PlaceCounter(n:integer):boolean;
    Procedure Initboard;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

{************* PlaceCounter ************}
Function TForm1.PlaceCounter(n:integer):boolean;
{The search routine - looks for a place for queen # n}
var
  i,j:integer;
  placed,r :boolean;
  prevboard:TBoard;

    Function RowIsClear(x:integer):boolean; {checks row OK}
    var
      i:integer;
    Begin
      result:=true;
      for i:= 1 to 8 do  if board[i,x]>0 then result:=false;
    end;

    Function ColIsClear(x:integer):boolean; {checks column OK}
    var
      i:integer;
    Begin
     result:=true;
     for i:= 1 to 8 do  if board[x,i]>0 then result:=false;
    end;

    Function DiagsAreClear(x,y:integer):boolean; {Check diagonals are OK}
    var
      i,j:integer;
    Begin
      result:=true;
      {go up and left}
      i:=x-1;
      j:=y-1;
      while (result=true) and (i>=1) and (j>=1)
      do  if board[i,j]>0 then result:=false
      else
      Begin
        dec(i);
        dec(j);
      end;
      {go up and right}
      i:=x+1;
      j:=y-1;
      while (result=true) and (i<=8) and (j>=1)
      do  if board[i,j]>0 then result:=false
      else
      Begin
        inc(i);
        dec(j);
      end;

      {go down and left}
      i:=x-1;
      j:=y+1;
      while (result=true) and (i>=1) and (j<=8)
      do  if board[i,j]>0 then result:=false
      else
      Begin
        dec(i);
        inc(j);
      end;

      {go down and right}
      i:=x+1;
      j:=y+1;
      while (result=true) and (i<=8) and (j<=8)
      do  if board[i,j]>0 then result:=false
      else
      Begin
        inc(i);
        inc(j);
      end;
    end;

Begin   {placecounter}
  inc(counter);
  prevboard:=board;
  placed:=false;
  i:=2;
  j:=1;
  while (i<=8) and (j<=8) and not placed do
  Begin
    r:=RowIsClear(j);
    if (board[i,j]=0)
       and r
       and ColIsClear(i)
       and DiagsAreClear(i,j)
    then
    Begin
      board[i,j]:=n;
      placed:=true;
    end
    else
    Begin
      if not r then i:=9 else inc(i);
      if i>8 then
      Begin
        i:=1;
        inc(j);
      end;
    end;
    if placed
    then  if (n<8) then placed:=placeCounter(n+1)
          else
    else board:=prevboard; {erase that move and continue search}
  end;
  result:=placed;
end;

{************ InitBoard **********}
Procedure TForm1.InitBoard;
{initialize the board}
var
  i,j:integer;
Begin
  for i:=1 to 8 do
  for j:= 1 to 8 do
    board[i,j]:=0;
  if typegrp.itemindex=1 then {plus version - rule out diagonals}
  for i:=1 to 8 do
  Begin
    board[i,i]:=-1;
    board[i,9-i]:=-1;
  end;
End;
{**************** SolveBtn ************}
procedure TForm1.SolveBtn(Sender: TObject);
{Search for a solution}
var
  i,j:integer;
begin
  initboard;
  counter:=0;
  If placecounter(1) then label2.caption:='Solved!'
  else label2.caption:='Sorry - no solution found';
  label1.caption:='Postions tried='+inttostr(counter);
  for i:=1 to 8 do
  for j:= 1 to 8 do
  with stringgrid1 do
  begin
    if board[i,j]=-1 then cells[i-1,j-1]:='X'
    else if board[i,j]>0 then cells[i-1,j-1]:=inttostr(board[i,j])
    else cells[i-1,j-1]:=' ';
  end;
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
  Initboard;
end;

procedure TForm1.StaticText1Click(Sender: TObject);
begin
  ShellExecute(Handle, 'open', 'http://www.delphiforfun.org/',
  nil, nil, SW_SHOWNORMAL) ;
end;

end.
